/*  DGTLCLK.C - A digital clock program

	This Windows 3.1 program is free for whoever wants it.  It displays
	a digital clock display in either big or tiny digits and floats on
	top of all windows.  It was written in Turbo C/C++ Visual Edition
	for Windows in the SMALL memory model. Link with DGTLCLK.DEF and
	DGTLCLK.RC...

	Date:
	08-24-94

	Author:
	Thomas R. Grubbe	Compuserve ID: 71042,151
*/

#define STRICT

#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "dgtlclk.h"

#define ID_TIMER	1

LRESULT CALLBACK _export WndProc(HWND, UINT, WPARAM, LPARAM);

void PaintDgtlClk(HWND hwnd, HDC hDC);
void PaintDigit(HDC hDC, int Pos, int Col, int Row);
void ReLoadBitmap(int Id);
void InitDgtlClk(void);
void DeInitDgtlClk(void);

#define HOUR	(systime->tm_hour)
#define MIN		(systime->tm_min)
#define SEC		(systime->tm_sec)

#define BIGWIDTH	13
#define BIGHEIGHT	22
#define TINYWIDTH	7
#define TINYHEIGHT	11

HINSTANCE ghInstance;
HWND ghwndMain;
HBITMAP hMainBitmap;
HDC hDCMem;
BITMAP bm;

int BmHeight;
int BmWidth;
int WindowWidth=0;
int WindowHeight=0;
int NumBitmaps = 9;
char szAppName[] = "Digital Clock";
int Hour, Minutes, Seconds;
int AmPm;			// 0=Doesn't apply, 1=AM, 2=PM;
int DisplayParam;	// Either IDM_BIGDIGITS or IDM_TINYDIGITS

// DGTLCLK.INI Profile Vars
int WindowLeft=0;
int WindowTop=0;
BOOL UseTinyDigits = FALSE;


#pragma argsused
int PASCAL WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
					LPSTR lpszCmd, int nCmdShow)
{
	HWND        hwnd;
	MSG         msg;
	WNDCLASS    wndclass;
	HANDLE 	 hAccel;

	ghInstance = hInstance;

	if (!hPrevInstance)
	{
		wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS |
								 WS_CAPTION | CS_BYTEALIGNCLIENT;
		wndclass.lpfnWndProc   = (WNDPROC)WndProc;
		wndclass.cbClsExtra    = 0;
		wndclass.cbWndExtra    = 0;
		wndclass.hInstance     = hInstance;
		wndclass.hIcon         = LoadIcon( NULL, "DGTLCLKICON");
		wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
		wndclass.hbrBackground = (HBRUSH)(GetStockObject(BLACK_BRUSH));
		wndclass.lpszMenuName  = NULL; // "MAINMENU";
		wndclass.lpszClassName = szAppName;

		RegisterClass (&wndclass);
	}

	InitDgtlClk();

    if (UseTinyDigits)	{
		BmWidth = TINYWIDTH;
		BmHeight = TINYHEIGHT;
	}
	else	{
		BmWidth = BIGWIDTH;
		BmHeight = BIGHEIGHT;
    }

	WindowWidth = BmWidth * NumBitmaps;
	WindowHeight = BmHeight;

	hwnd = CreateWindowEx(0, szAppName, szAppName,
		WS_POPUPWINDOW,
		WindowLeft,
		WindowTop,
		WindowWidth,
		WindowHeight,
		NULL,
		NULL,
		hInstance,
		NULL);

	ghwndMain = hwnd;

	SetWindowPos(hwnd, HWND_TOPMOST, WindowLeft, WindowTop,
				 WindowWidth, WindowHeight, SWP_SHOWWINDOW);

	if (!SetTimer(hwnd, ID_TIMER, 1000, NULL))	{
		MessageBox(hwnd, "Too many clocks or timers!", szAppName,
				   MB_ICONEXCLAMATION | MB_OK);
		return FALSE;
    }

	ShowWindow (hwnd, nCmdShow);
	UpdateWindow (hwnd);

	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg);
		DispatchMessage (&msg);
	}
	return msg.wParam;
}


LRESULT CALLBACK _export WndProc (HWND hwnd, UINT message,
								  WPARAM wParam, LPARAM lParam)
{
	static HMENU hMenu;

	switch (message)
	{
		case WM_CREATE:
		{
			HDC hDC;

			hMenu = LoadMenu(ghInstance, "MAINMENU");
			hMenu = GetSubMenu(hMenu, 0);

            if (UseTinyDigits)	{
				hMainBitmap = LoadBitmap(ghInstance, "ID_DIGITSTINY");
				CheckMenuItem(hMenu, IDM_TINYDIGITS, MF_CHECKED);
                DisplayParam = IDM_TINYDIGITS;
            }
			else	{
				hMainBitmap = LoadBitmap(ghInstance, "ID_DIGITSBIG");
				CheckMenuItem(hMenu, IDM_BIGDIGITS, MF_CHECKED);
                DisplayParam = IDM_BIGDIGITS;
            }
			GetObject(hMainBitmap, sizeof(BITMAP), &bm);

			hDC = GetDC(hwnd);
			hDCMem = CreateCompatibleDC(hDC);

			ReleaseDC(hwnd, hDC);
			InvalidateRect(hwnd, NULL, TRUE);            
		}
			return 0L;

		case WM_TIMER:
			InvalidateRect(hwnd, NULL, FALSE);
			return 0L;
        case WM_NCLBUTTONDBLCLK:
		case WM_NCRBUTTONDOWN:	// Pull down menu
		{
			RECT r, r2;

			GetWindowRect(hwnd, &r);
			GetClientRect(hwnd, &r2);
			TrackPopupMenu(hMenu, 0, r.left, r.top + r2.bottom, 0, hwnd, &r);
			return 0L;
		}
		case WM_NCHITTEST:		// Simulate caption hit
			if (!wParam)
				return HTCAPTION;
			else break;
		case WM_MOVE:
		case WM_SIZE:
		{
			RECT r;
            int BdrH, BdrW, cxScreen, cyScreen;

			GetWindowRect(hwnd, &r);

			WindowLeft = r.left;
			WindowTop = r.top;

			BdrH = GetSystemMetrics(SM_CYBORDER);
			BdrW = GetSystemMetrics(SM_CXBORDER);
			cxScreen = GetSystemMetrics(SM_CXSCREEN);
            cyScreen = GetSystemMetrics(SM_CYSCREEN);
			WindowHeight = BmHeight + (BdrH * 2);
			WindowWidth  = (BmWidth * NumBitmaps) + (BdrW * 2);

            if (WindowLeft < 0) WindowLeft = 0;
			if (WindowLeft > cxScreen) WindowLeft = cxScreen - WindowWidth;
			if (WindowTop < 0) WindowTop = 0;
            if (WindowTop > cyScreen) WindowTop = cyScreen - WindowHeight;

			MoveWindow(hwnd, WindowLeft, WindowTop, WindowWidth, WindowHeight, TRUE);
        }
            return 0L;
		case WM_PAINT:
		{
			HDC hDC;
			PAINTSTRUCT ps;

			hDC = BeginPaint(hwnd, &ps);
            PaintDgtlClk(hwnd, hDC);
			EndPaint(hwnd, &ps);
        }
		return 0L;
		case WM_COMMAND:
			switch (wParam)
			{
				case IDM_TINYDIGITS:
				case IDM_BIGDIGITS:
					CheckMenuItem(hMenu, DisplayParam, MF_UNCHECKED);
					DisplayParam = wParam;
					CheckMenuItem(hMenu, DisplayParam, MF_CHECKED);
					ReLoadBitmap(wParam);
                    return 0L;
				case IDM_EXIT:
					SendMessage(hwnd, WM_CLOSE, 0, 0L);
					break;
			}
			return 0L;
        case WM_ENDSESSION:
		case WM_CLOSE:
		case WM_DESTROY:
			DeInitDgtlClk();
        	DeleteDC(hDCMem);
        	DeleteObject(hMainBitmap);
			DestroyMenu(hMenu);
			KillTimer(hwnd, ID_TIMER);
			PostQuitMessage (0) ;
			return 0 ;
	}
	return DefWindowProc (hwnd, message, wParam, lParam) ;
}


// Toggle between big or tiny digit bitmaps
//
void ReLoadBitmap(int Id)
{
	HDC hDC;

	DeleteDC(hDCMem);
	DeleteObject(hMainBitmap);
    hMainBitmap = 0;

	if (Id == IDM_TINYDIGITS)	{
    	UseTinyDigits = TRUE;
		hMainBitmap = LoadBitmap(ghInstance, "ID_DIGITSTINY");
		DisplayParam = IDM_TINYDIGITS;
		BmWidth = TINYWIDTH;
		BmHeight = TINYHEIGHT;
	}
	else	{
    	UseTinyDigits = FALSE;
		hMainBitmap = LoadBitmap(ghInstance, "ID_DIGITSBIG");
		DisplayParam = IDM_BIGDIGITS;
		BmWidth = BIGWIDTH;
		BmHeight = BIGHEIGHT;
	}
	GetObject(hMainBitmap, sizeof(BITMAP), &bm);

	hDC = GetDC(ghwndMain);
	hDCMem = CreateCompatibleDC(hDC);

	ReleaseDC(ghwndMain, hDC);

	WindowWidth = BmWidth * NumBitmaps;
	WindowHeight = BmHeight;

	MoveWindow(ghwndMain, WindowLeft, WindowTop, WindowWidth, WindowHeight, TRUE);
}


// Get the time and call PaintDigit() to do the drawing
//
#pragma argsused
void PaintDgtlClk(HWND hwnd, HDC hDC)
{
	char Buffer[80];
	time_t	lTime;
	struct tm	*systime;
    int Row, Col, Pos;
	register int i;
    BOOL AM = FALSE;

	time(&lTime);
	systime = localtime(&lTime);

	Hour = (HOUR % 12) ? (HOUR % 12) : 12;
	Minutes = MIN;
	Seconds = SEC;
	AmPm = (HOUR / 12) + 1;

	if (AmPm == 1)
    	AM = TRUE;
	sprintf(Buffer, "%d:%02d:%02d", Hour, Minutes, Seconds); 

	// Beleive or not, this is faster than painting digits within
	// a for() loop with several nested conditional tests to slow
	// things down.
    Pos = 0;
	if (Hour < 10)	{
    	NumBitmaps = 8;
		PaintDigit(hDC, Pos++, 0, Buffer[0] - '0');
		PaintDigit(hDC, Pos++, 1, 1);
		PaintDigit(hDC, Pos++, 0, Buffer[2] - '0');
		PaintDigit(hDC, Pos++, 0, Buffer[3] - '0');
		PaintDigit(hDC, Pos++, 1, 1);
		PaintDigit(hDC, Pos++, 0, Buffer[5] - '0');
		PaintDigit(hDC, Pos++, 0, Buffer[6] - '0');
	}
	else	{
    	NumBitmaps = 9;
		PaintDigit(hDC, Pos++, 0, Buffer[0] - '0');
		PaintDigit(hDC, Pos++, 0, Buffer[1] - '0');
		PaintDigit(hDC, Pos++, 1, 1);
		PaintDigit(hDC, Pos++, 0, Buffer[3] - '0');
		PaintDigit(hDC, Pos++, 0, Buffer[4] - '0');
		PaintDigit(hDC, Pos++, 1, 1);
		PaintDigit(hDC, Pos++, 0, Buffer[6] - '0');
		PaintDigit(hDC, Pos++, 0, Buffer[7] - '0');
    }

	if (AM)
		PaintDigit(hDC, Pos, 1, 2);
    else
		PaintDigit(hDC, Pos, 1, 3);

    SendMessage(ghwndMain, WM_MOVE, 0, 0L);
}


// BitBlt the bitmap digit at Row,Col of hMainBitmap
//
void PaintDigit(HDC hDC, int Pos, int Col, int Row)
{
	HBITMAP hbmpOld;

    hbmpOld = SelectObject(hDCMem, hMainBitmap);
	BitBlt(hDC, Pos*BmWidth, 0, BmWidth, BmHeight, hDCMem, Col*BmWidth, Row*BmHeight, SRCCOPY);
	SelectObject(hDCMem, hbmpOld);
}


// Get DGTLCLK.INI private profile vars.
//
void InitDgtlClk(void)
{
	char WindowsDir[128];
	char INIPathName[128];

	GetWindowsDirectory((LPSTR)WindowsDir, sizeof(WindowsDir));
	if (WindowsDir[strlen(WindowsDir)-1] != '\\')
		strcat(WindowsDir, "\\");
	sprintf(INIPathName, "%s%s", WindowsDir, "DGTLCLK.INI");

	WindowLeft = (int)GetPrivateProfileInt("Defaults", "WindowLeft", 0, (LPSTR)INIPathName);
	WindowTop = (int)GetPrivateProfileInt("Defaults", "WindowTop", 0, (LPSTR)INIPathName);
	UseTinyDigits = (BOOL)GetPrivateProfileInt("Defaults", "UseTinyDigits", 0, (LPSTR)INIPathName);
}

// Write DGTLCLK.INI private profile vars.
//
void DeInitDgtlClk(void)
{
	char WindowsDir[128];
	char INIPathName[128];
    char num[18];
    RECT r;

	GetWindowsDirectory((LPSTR)WindowsDir, sizeof(WindowsDir));
	if (WindowsDir[strlen(WindowsDir)-1] != '\\')
		strcat(WindowsDir, "\\");
	sprintf(INIPathName, "%s%s", WindowsDir, "DGTLCLK.INI");

	GetWindowRect(ghwndMain, &r);
	WritePrivateProfileString("Defaults", "WindowLeft",
							  (LPSTR)itoa(r.left, num, 10),
							  (LPSTR)INIPathName);
	WritePrivateProfileString("Defaults", "WindowTop",
							  (LPSTR)itoa(r.top, num, 10),
							  (LPSTR)INIPathName);
	WritePrivateProfileString("Defaults", "UseTinyDigits",
							  (LPSTR)itoa((int)UseTinyDigits, num, 10),
							  (LPSTR)INIPathName);
}